"use_strict";
function load_preview_image(image, src){
  let type=image.data('type');
  if(type&&type.match(/^image\/(png|jpeg|jpg|gif)/)){
    let img=document.createElement('img');
    img.onload=function(e){resize_image(image,img)};
    img.src=src;
  }else{
    image.attr('src',src);
    $('div.controls',image.closest('div.preview-image-content').show()).hide();
  }
}

function resize_image(image, img)
{
  const maxw=parseInt(image.css('max-width')),maxh=parseInt(image.css('max-height'));
  let w=img.width,h=img.height,resize=true,canvas=document.createElement('canvas'), ctx=canvas.getContext('2d');
  if(w>h&&w>maxw) {
    canvas.width = maxw;
    canvas.height = canvas.width * (h / w);
  } else if(h>maxh) {
    canvas.height = maxh;
    canvas.width = canvas.height * (w / h);
  } else {
    canvas.width = w;
    canvas.height = h;
    resize = false;
  }
  ctx.drawImage(img, 0, 0, w, h, 0, 0, canvas.width, canvas.height);
  image.attr('src', canvas.toDataURL(image.data('type'))).closest('div.preview-image-content').show();
}

const multiplier = Math.PI / 180;
function rotate_image(image, degree)
{
  let canvas = document.createElement('canvas'), contxt=canvas.getContext('2d'), r_deg=Math.floor(degree) % 360,current=image.dataset.rdeg||0;
  current = Number(current);
  r_deg = r_deg - (r_deg % 90);
  switch(r_deg) {
    case 90:
    case -270:
      {
        canvas.setAttribute('width', image.height);
        canvas.setAttribute('height', image.width);
        contxt.rotate(r_deg * multiplier);
        contxt.drawImage(image, 0, -image.height);
        current = (current - 270) % 360;
        break;
      }

    case -180:
    case 180:
      {
        canvas.setAttribute('width', image.width);
        canvas.setAttribute('height', image.height);
        contxt.rotate(r_deg * multiplier);
        contxt.drawImage(image, -image.width, -image.height);
        current = (current - 180) % 360;
        break;
      }

    case -90:
    case 270:
      {
        canvas.setAttribute('width', image.height);
        canvas.setAttribute('height', image.width);
        contxt.rotate(r_deg * multiplier);
        contxt.drawImage(image, -image.width, 0);
        current = (current - 90) % 360;
        break;
      }

    case 0:
    default:
      {
        canvas.setAttribute('width', image.width);
        canvas.setAttribute('height', image.height);
        contxt.rotate(r_deg * multiplier);
        contxt.drawImage(image, 0, 0);
        break;
      }
  }
  image.dataset.rdeg = current;
  image.src=canvas.toDataURL(image.dataset.type);
  return -current;
}
$(document).on('click', 'div.preview-image-content div.controls button.rl', function(ev){
  var e=$(this).parent().parent(),img=e.find('img.preview-image')[0],input=e.find('input.rotate')[0];
  ev.preventDefault();
  input.value = rotate_image(img,-90);
  return false;
});
$(document).on('click', 'div.preview-image-content div.controls button.rr', function(ev){
  var e=$(this).parent().parent(),img=e.find('img.preview-image')[0],input=e.find('input.rotate')[0];
  ev.preventDefault();
  input.value = rotate_image(img,90);
  return false;
});

