// https://www.saltycrane.com/blog/2014/09/bank-style-session-timeout-example-using-jquery-bootstrap-and-flask/
// Modified to be pure Javascript (no jQuery)
const sessionMonitor = function(options) {
	"use strict";
	var $d=document,$w=window,wrap=function(a,id){var e=$d.createElement('div');if(id){e.id=id;}e.appendChild(a);return e;},defaults = {
		// Session lifetime (milliseconds)
		sessionLifetime: 15 * 60 * 1000,
		// Amount of time before session expiration when the warning is shown (milliseconds)
		timeBeforeWarning: 2 * 60 * 1000,
		// Minimum time between pings to the server (milliseconds)
		minPingInterval: 1 * 60 * 1000,
		// Space-separated list of events that indicate a user is active
		activityEvents: 'mouseup',
		// Name of the session cookie to handle multiple tabs
		cookieName: 'ss_bo_session',
		// URL to ping the server using HTTP POST to extend the session
		pingUrl: '/ping',
		// Parameters for the ping
		pingParams: {},
		// URL used to log out when the user clicks a "Log out" button
		logoutUrl: '/logout',
		// URL used to log out when the session times out
		timeoutUrl: '/logout?timeout=1',
		// Text to use for the warning pop-up (replaced by template)
		textLogOut: 'Log Out',
		textStayLoggedIn: 'Stay Logged In',
		//   The text can have the number of minutes until session expires with: [MINUTES]
		warnPopupText: 'Your session will expire in [MINUTES] minutes.',
		// ID for HTML Template to use for the warning pop-up
		//   The design of the template should include 2 buttons/links with these IDs:
		//     session-monitor-stay-logged-in
		//     session-monitor-log-out
		//   The text can have the number of minutes until session expires with: [MINUTES]
		warningTemplate:'',
		// Functions
		ping: async function() {
			// Ping the server to extend the session expiration using a POST request.
			try{let d=[],t=this,x=t.pingParams;for(const i in x){d.push(encodeURIComponent(i) + '=' + encodeURIComponent(x[i]));}d=d.join('&');await fetch(t.pingUrl, {method:'POST',cache:'no-cache',redirect:'error',referrerPolicy:'origin-when-cross-origin',headers:{'Content-type':'application/x-www-form-urlencoded'},body:d}).then(function(r){r.json()});}catch(e){/*console.error(e);t.logout.apply(t);*/}
		},
		logout: function() {
			// Go to the logout page.
			$w.location.href = this.logoutUrl;
		},
		ontimeout: function() {
			// Go to the timeout page.
			$w.location.href = this.timeoutUrl;
		},
		onwarning: function() {
			var warningMinutes = Math.round(this.timeBeforeWarning / 60 / 1000),e,a = $d.querySelector('#session-warn-overlay'),t=this;
			if(!a) {
				a=$d.createElement('div');
				a.id='session-warning';
				a.innerHTML=(t.warningTemplate&&(e=$d.getElementById(t.warningTemplate))&&e.innerText.replace(/\[MINUTES\]/g,warningMinutes))||(t.warnPopupText.replace(/\[MINUTES\]/g,warningMinutes)+'<br><div type="button" class="button" id="session-monitor-stay-logged-in">'+t.textStayLoggedIn+'</div>&nbsp;<div type="button" class="button" id="session-monitor-log-out">'+t.textLogOut+'</div>');
				a=wrap(wrap(a,'session-warn-content'), 'session-warn-overlay');
				$d.body.appendChild(a);
				//a = $d.querySelector('#session-warning');
				a.querySelector('#session-monitor-stay-logged-in').addEventListener('click', function(e){e.preventDefault();t.extend_session();a.remove();});
				a.querySelector('#session-monitor-log-out').addEventListener('click', function(e){t.logout.apply(t)});
			}
		},
		onbeforetimeout: async function() {
			// By default this does nothing. Override this method to perform actions
			// (such as saving draft data) before the user is automatically logged out.
			// This may optionally return a Promise, in which case ontimeout will be
			// executed when the promise is resolved or rejected.
			return true;
		},
	},
	self = {},
	_now=function(){return(new Date()).getTime()},
	_getSession=function(){if(self.cookieName){let s;try{s=JSON.parse((localStorage&&localStorage[self.cookieName])||get_cookie(self.cookieName))||{};}catch(e){s=false;}return s;}else{return false}},
	_setSession=function(){if(self.cookieName){let s=JSON.stringify(Object.assign(_getSession()||{},{'lastPingTime':_lastPingTime}));if(localStorage){localStorage[self.cookieName]=s}else{set_cookie(self.cookieName,s)}}},
	_clearSession=function(){if(self.cookieName){let s=_getSession()||{};delete s.lastPingTime;s=JSON.stringify(s);if(localStorage){localStorage[self.cookieName]=s}else{set_cookie(self.cookieName,s)}}},
	_checkSession=function(){let s=_getSession()||{lastPingTime:0};if(s.lastPingTime>_now()){_setSession();} else if(s.lastPingTime>_lastPingTime){_lastPingTime=s.lastPingTime;_resetTimers();return true;}return false;},
	_warningShown=false,
	_warningTimeoutID,
	_expirationTimeoutID,
	// The time of the last ping to the server.
	_lastPingTime = 0;

	function extend_session() {
		// Extend the session expiration. Ping the server and reset the timers if
		// the minimum interval has passed since the last ping or the warning was
		// shown.
		_checkSession();
		var now = _now(),
		timeSinceLastPing = now - _lastPingTime;

		if (timeSinceLastPing > self.minPingInterval || _warningShown) {
			_lastPingTime = now;
			_setSession();
			_resetTimers();
			self.ping.apply(self);
			_warningShown=false;
		}
	}

	function _resetTimers() {
		// Reset the session warning and session expiration timers.
		var now=_now(), sincePing=now-_lastPingTime, warningTimeout = self.sessionLifetime - self.timeBeforeWarning;
		if(warningTimeout-sincePing <= 0){_lastPingTime=now;_setSession();sincePing=0;}

		$w.clearTimeout(_warningTimeoutID);
		$w.clearTimeout(_expirationTimeoutID);
		_warningTimeoutID = $w.setTimeout(_onWarning, warningTimeout-sincePing);
		_expirationTimeoutID = $w.setTimeout(_onTimeout, self.sessionLifetime-sincePing);
	}

	function _onWarning(){
		if(!_checkSession()){
			_warningShown=true;
			self.onwarning.apply(self);
		}
	}

	async function _onTimeout() {
		// A wrapper that calls onbeforetimeout and ontimeout and supports asynchronous code.
		if(!_checkSession()){
			try{_clearSession();await self.onbeforetimeout.apply(self);}
			catch(e){console.error(e);}
			finally{self.ontimeout.apply(self);}
		}
	}

	// Add default variables and methods, user specified options, and non-overridable
	// public methods to the session monitor instance.
	self=Object.assign(self,defaults,options,{extend_session: extend_session});
	// Set an event handler to extend the session upon user activity (e.g. mouseup).
	self.activityEvents.split(' ').forEach(function(a){$d.addEventListener(a,function(){!_warningShown&&self.extend_session()})});
	// Start the timers and ping the server to ensure they are in sync with the backend session expiration.
	self.extend_session();

	return self;
};
